% NSGSA   Non-dominated Sorting Gravitational Search Algorithm
% Version 1.0
% Released on Aug. 12th, 2011 (1390/5/20)
% by  Hadi Nobahari and Mahdi Nikusokhan
%
% NSGSA is a multi-objective optimization algorithm based on the 
% Gravitaional Search Algorithm
%
%
%
% [FOpt,XOpt]=NSGSA(FunctionName,low,up,MaxFE,Nlist) solves the
% multi-objective optimization problem formulated in FunctionName.
%
% FunctionName: A string containing the name of the MATLAB m-file in which
% the objective functions are implemented in. This function should have the
% following form:
%
%    Y=MyFunctionName(X)
%
% Where X is row vector of N elements corresponding to a point in
% search space for which the value of the objective functions are
% calculated and assigned to Y. Y is a row vector of M elements
% corresponding to a point within the objective space.
%
% low: A row vector of N elements that specifies the minimum limits of
% the search variables. The i-th column corresponds to the i-th variable.
%
% up: A row vector of N elements that specifies the maximum limits of
% the search variables. The i-th column corresponds to the i-th variable.
%
% MaxFE: Maximum number of function evaluations. The algorithm stops when
% it has executed the objective function for MaxFE times.
%
% Nlist: The length of external archive that it is the maximum number of
% points in Pareto front, returned by NSGSA.
%
%
%
% [FOpt,XOpt]=NSGSA(FunctionName,low,up,MaxFE,Nlist,Options) lets you to
% set the internal parameters of NSGSA though "Options" argument.
%
% Options : A cell array containing the user-defined values for the control
% parameters of the algorithm. In order to use the default parameter
% setting (which is based on the problem dimension), this argument must be
% absent. However, when NSGSA is called with argument Options, its content will
% be interpreted as the following pattern and description of the
% parameters:
%
%   {mass_num, Nelit, beta, SignMut, PosMut, ReorderMut, w0, w1}
%
% mass_num : Swarm size.
% Nelit : Percent of elitism. It should be between 0 and 1.
% beta: Coefficient of the search interval.
% SignMut: Sign mutaion probability. It should be between 0 and 1.
% PosMut: Uniform mutaion probability. It should be between 0 and 1.
% ReorderMut: Reordering mutaion probability. It should be between 0 and 1.
% w0: Initial value of the inertial coefficient.
% w1: Final value of the inertial coefficient.
%
% XOpt : A column vector containing the optimum values of the variables
% which will minimize the objective function.
%
% FOpt : Optimum objective function value found; associated with XOpts.
%
% Here is an Example of how to use NSGSA:
% Suppose that we want to find the minimum of the 2-variable and 3-objective function
%       f1(x,y) = 0.5*(x^2+y^2)+sin(x^2+y^2)
%       f2(x,y) = (3*x-2*y+4)^2/8+(x-y+1)^2/27+15
%       f3(x,y) = 1/(x^2+y^2+1)-1.1*exp(-(x^2+y^2))
% where  -3<x<3 and -3<y<3.
%
% The first step is to write the proper MATLAB function. For this case we
% should write a small MATLAB function named MyFunction.m that will look like this:
%
% function Y=MyFunction(X)
% 
%     x=X(1);y=X(2);
%     Y(1)=0.5*(x^2+y^2)+sin(x^2+y^2);
%     Y(2)=(3*x-2*y+4)^2/8+(x-y+1)^2/27+15;
%     Y(3)=1/(x^2+y^2+1)-1.1*exp(-(x^2+y^2));
% 
% return
%
% The next step is calling NSGSA with appropriate input arguments. If we
% wish to use default parameter setting, the following command should be
% entered at MATLAB command window:
%
%     [FOpt,XOpt]=NSGSA('MyFunction',[-3 -3],[3 3],2100,800); 
%
% And to run the algorithm with custom settings, say with 120 swarm
% size, 0.7 for percent of elitism, 0.2 coefficient of search interval, 0.9
% sign mutation probability, 0.01 uniform mutation probability, 0
% reordering mutation probability, 0.9 initial value of inertial
% coefficient, and 0.5 initial value of inertial coefficient
% Roulette method, WeightingFactor of 0.75, and PCAFactor of 3, the following
% command should be used:
%
%     [FOpt,XOpt]=NSGSA('MyFunction',[-3 -3],[3 3],2100,800,{120,0.7,0.2,0.9,0.01,0,0.9,0.5});
%
%The output will be something like this:
%
% XOpt = <762x2 double>
%
% FOpt = <762x3 double>
%
% for more details on Non-dominated Sorting Gravitational Search Algorithm
% (NSGSA), you can refer to:
% H. Nobahari and D. Darabi, 'Non-dominated Sorting Gravitational Search Algorithm', 
% ICSI2011, International Conference on Swarm Intelligence, June, 14-15, 2011, 
% Cergy, France (2011)
% Paper Link: http://icsi11.eisti.fr/papers/paper_12.pdf

function [fbest,Lbest]=NSGSA(FunctionName,low,up,MaxFE,Nlist,Option)

if nargin==5
    Options.mass_num=100; 
    Options.Nelit=0.5;
    Options.beta=0.2;
    Options.SignMut=0.9; 
    Options.PosMut=0.0; 
    Options.ReorderMut=0.0; 
    Options.w0=0.9;
    Options.w1=0.5;
else
    Options.mass_num=Option{1}; 
    Options.Nelit=Option{2}; 
    Options.beta=Option{3}; 
    Options.SignMut=Option{4}; 
    Options.PosMut=Option{5}; 
    Options.ReorderMut=Option{6}; 
    Options.w0=Option{7}; 
    Options.w1=Option{8}; 
end;

Nelit=Options.Nelit;
mass_num=Options.mass_num;
max_it=round(MaxFE/Options.mass_num); % maximum iteration

% Initialization ----------------------------------------
dim=length(low);
for i=1:dim
    loc(i,:)=rand(1,mass_num).*(up(i)-low(i))+low(i);
end

fbest=[];Lbest=[];
V=zeros(dim,mass_num);
% Iteration Loop ----------------------------------------
for iteration=1:max_it
% Function Evaluation------------------------------------    
    for i=1:mass_num 
        L=loc(:,i)';
        fitness(i,:)=feval(FunctionName,L);
    end
% Selecting and Ranking the moving particles -------------        
    [mapfit Lbest fbest loc V]=Ranking(fitness,Lbest,fbest,loc,V,mass_num,Nelit,Nlist,iteration,max_it);
% Mass update of each particle ---------------------------        
    MG=mass_calculate(mapfit);
% Gravity constant update --------------------------------
    G=Gconstant(iteration,Options,low,up,max_it);
% Acceleration update ------------------------------------
    E=Gfield(MG,loc,max_it,iteration,mass_num,dim);
% Velocity and position update ---------------------------
    [loc,V]=move(loc,E,G,V,iteration,max_it,Options,up,low);
    Disp=strcat('Iter=  ',strcat(Num2Str(iteration)),' of ,',strcat(Num2Str(max_it)));
    disp(Disp);
 
end %iteration
%==========================================================================
% FUNCTIONS
%==========================================================================
function G=Gconstant(iteration,Options,low,up,max_it)

persistent G0

beta=Options.beta;

ratio=0.99;
alfa=0.8;
if iteration==1,
   G0=beta*max(up-low);
   G=G0;
else
   if iteration<=(ratio*max_it)
       G0=G0*(1-(iteration+1)/max_it)/(1-iteration/max_it);
       G=G0;
   else
       G0=alfa.*G0;
       G=G0;
   end
end
%==========================================================================
function E=Gfield(MG,loc,max_it,iteration,mass_num,dim);

final_per=100;
init_per=2;
c2=(init_per+(1-iteration/max_it)*(final_per-init_per));
c2=round(mass_num*c2/100);

[MGs ds]=sort(MG,'descend'); 
for i=1:mass_num
    E(:,i)=zeros(dim,1);
    for ii=1:c2
        j=ds(ii);
        if j~=i
            for k=1:dim 
                R=norm(loc(:,i)-loc(:,j),1);
                E(k,i)=E(k,i)+rand*(MG(j))*((loc(k,j)-loc(k,i))/(R+eps));
            end
        end
    end
end
%==========================================================================
function MG=mass_calculate(fit);
Fmax=max(fit);    Fmin=min(fit);
[c i]=size(fit);
if Fmax==Fmin
    MG=mf.*ones(c,1);
else
    MG=(Fmax-fit)./(Fmax-Fmin);
end

MG=MG./sum(MG);
%==========================================================================
function [loc,V]=move(loc,E,G,V,iteration,max_it,Options,up,low)


[dim,c]=size(loc);


V=(Options.w0-Options.w1*iteration/max_it)*V+G*E;

% velocity mutation
SignMut=Options.SignMut;
MutRate=SignMut;
if mod(iteration,2)~=0,
    MutRate=0;
end
MutMatrix=reshape(randperm(c*dim),dim,c);
MUT=(MutMatrix<=ceil(MutRate*c*dim));

% Reordering mutation
ReorderMut=Options.ReorderMut;
MutRate=ReorderMut; %*rand;
if mod(iteration,2)==0,
    MutRate=0;
end
MutMatrix=randperm(c);
MUTc=(MutMatrix<=ceil(MutRate*c));
MUTr=[];
for i=1:c,
    row=randperm(dim);
    Vr(:,i)=V(row,i);
    MUTr=[MUTr MUTc(i)*ones(dim,1)];
end

Vtemp=(1-MUTr).*V-MUTr.*Vr; %reordering mutation
loc=loc+(1-MUT).*Vtemp-MUT.*Vtemp; %only position mutation

% position mutation
MutRate1=Options.PosMut;
MutMatrix=reshape(randperm(c*dim),dim,c);
MUT1=(MutMatrix<=ceil(MutRate1*c*dim));
RandMat=zeros(dim,c);
for i=1:c,
    RandMat(:,i)=rand(dim,1).*(up'-low')+low';
end

loc=(1-MUT1).*loc+MUT1.*RandMat;
[loc V]=fit_bound(loc,V,up,low);
%==========================================================================
function  [loc V]=fit_bound(loc,V,up,low)

[dim,mass_num]=size(loc);
for i=1:mass_num 
    Tp=loc(:,i)>up';Tm=loc(:,i)<low';
    loc(:,i)=(loc(:,i).*(~(Tp+Tm)))+up'.*Tp+low'.*Tm; 
    V(:,i)=(V(:,i).*(~(Tp+Tm)))-V(:,i).*Tp-V(:,i).*Tm;
end
%==========================================================================
function [fbest Lbest freject Lreject Lmax Lbound listCounter Sreject area]=ParetoArchive(f,loc,fbest,Lbest,Nlist,listCounter,dimF),
fbestTemp=fbest; LbestTemp=Lbest;
freject=[]; Lreject=[]; Lmax=[]; Lbound=[]; area=[];
Sreject=1;
% 0: New
% 1: Old
areaType=3;
% 1: based on area
% 2: based on Euclidian distance
% 3: based on spread merit
FirstIter=0;
if isempty(fbest),
    fbest=f;
    Lbest=loc;
    listCounter=1;
    FirstIter=1;
end
countRej=0; replace=0; pareCheck=1; 
i=1;
while i<=listCounter,
    if dominate(fbest(i,:),f) || all(fbest(i,:)==f),
        countRej=countRej+1;
        Sreject=0;
        freject=f;
        Lreject=loc;
    elseif dominate(f,fbest(i,:))
        freject=[freject;fbest(i,:)];
        Lreject=[Lreject;Lbest(i,:)];
        if pareCheck==1,
            fbest(i,:)=f;
            Lbest(i,:)=loc;
        else
            clear fbestTemp1 LbestTemp1
            fbestTemp1=[fbest(1:(i-1),:);fbest((i+1):listCounter,:)];
            LbestTemp1=[Lbest(1:(i-1),:);Lbest((i+1):listCounter,:)];
            fbest=fbestTemp1;
            Lbest=LbestTemp1;
            listCounter=listCounter-1;
        end
        pareCheck=2;
        replace=1;
    end
    i=i+1;
end
if replace==0 && countRej==0,
    if listCounter<=Nlist,
        fbest(listCounter+1,:)=f;
        Lbest(listCounter+1,:)=loc;
        listCounter=listCounter+1;
    else
        boundExt=0;
        if dimF==2,
            if f(1)<=min(fbest(:,1)) || f(2)<=min(fbest(:,2)),
                boundExt=1; %extention is applied
            end
        elseif dimF==3,
            if f(1)<=min(fbest(:,1)) || f(2)<=min(fbest(:,2)) || f(3)<=min(fbest(:,3)),
                boundExt=1; %extention is applied
            end
        end
        % insert f to fbest
        fbest=[fbest;f];
        Lbest=[Lbest;loc];
        [value index]=sort(fbest(:,1));
        sortedFbest=fbest(index,:);
        sortedLbest=Lbest(index,:);
        fbest=sortedFbest;
        Lbest=sortedLbest;
        [fbestNum,dimF]=size(fbest);% fbestNum: size of fbest + 1
        fbest1=fbest(1:(fbestNum-1),:);
        fbest2=fbest(2:fbestNum,:);
        deltaf=abs(fbest2-fbest1);
        if areaType==1,
            area=ones(fbestNum-1,1);
            for i=1:dimF,
                area=deltaf(:,i).*area;
            end
        else
            area=zeros(fbestNum-1,1);
            for i=1:dimF,
                area=deltaf(:,i).^2+area;
            end
        end
        [MaxArea indexMax]=max(area); 
        [MinArea indexMin]=min(area); %indexMin: index of minimum area
        DiffAreaOld=MaxArea-MinArea;
        if areaType==3,
            areaBar=mean(area);
            DiffAreaOld=sum(area-areaBar)/((fbestNum-1)*areaBar);
        end
        for i=1:2,
            ftemp(i,:,:)=[fbest(1:(indexMin-2+i),:) ; fbest((indexMin+i):fbestNum,:)];
            [fbestNum1,dimX]=size(Lbest);
            if dimX==1,
                Ltemp(i,:,:)=[Lbest(1:(indexMin-2+i),:) ; Lbest((indexMin+i):fbestNum,:)]';
            else
                Ltemp(i,:,:)=[Lbest(1:(indexMin-2+i),:) ; Lbest((indexMin+i):fbestNum,:)];
            end
            ftemp1(i,:,:)=ftemp(i,1:(fbestNum-2),:);
            ftemp2(i,:,:)=ftemp(i,2:(fbestNum-1),:);
            deltaFtemp(i,:,:)=abs(ftemp2(i,:,:)-ftemp1(i,:,:));
            if areaType==1,
                areaTemp=ones(fbestNum-2,1);
                for j=1:dimF,
                    areaTemp=squeeze(deltaFtemp(i,:,j))'.*areaTemp;
                end
            else
                areaTemp=zeros(fbestNum-2,1);
                for j=1:dimF,
                    areaTemp=squeeze(deltaFtemp(i,:,j))'.^2+areaTemp;
                end
            end
            IndexRemove(i)=indexMin-1+i; % IndexRemove: index of removed point
            [MaxRemove(i) indexMaxRemove(i)]=max(areaTemp); 
            [MinRemove(i) indexMinRemove(i)]=min(areaTemp); % indexMinRemove: index of minimum area after removing Removed point
            if areaType==3,
                areaBar=mean(areaTemp);
                MaxRemove(i)=sum(areaTemp-areaBar)/((fbestNum-1)*areaBar);
                MinRemove(i)=0;
            end
        end
        [DiffAreaSorted,indexDiffAreaSorted]=sort(MaxRemove-MinRemove);
        IndexRemove=IndexRemove(indexDiffAreaSorted);
        find=1;
        if boundExt==1,
            for i=1:2,
                if IndexRemove(indexDiffAreaSorted(i))~=1 && IndexRemove(indexDiffAreaSorted(i))~=fbestNum,                    
                    freject=squeeze(fbest(IndexRemove(indexDiffAreaSorted(i)),:,:));
                    Lreject=squeeze(Lbest(IndexRemove(indexDiffAreaSorted(i)),:,:));
                    clear fbest Lbest
                    fbest=squeeze(ftemp(indexDiffAreaSorted(i),:,:));
                    Lbest=squeeze(Ltemp(indexDiffAreaSorted(i),:,:));
                    find=0;    break
                end
            end
        else
            for i=1:2,
                if DiffAreaSorted(i)<DiffAreaOld && IndexRemove(indexDiffAreaSorted(i))~=1 && IndexRemove(indexDiffAreaSorted(i))~=fbestNum,
                    freject=squeeze(fbest(IndexRemove(indexDiffAreaSorted(i)),:,:));
                    Lreject=squeeze(Lbest(IndexRemove(indexDiffAreaSorted(i)),:,:));
                    clear fbest Lbest
                    fbest=squeeze(ftemp(indexDiffAreaSorted(i),:,:));
                    Lbest=squeeze(Ltemp(indexDiffAreaSorted(i),:,:));
                    find=0;    break
                end
            end
        end
        
        if find==1,
            Sreject=0;
            fbest=fbestTemp;
            Lbest=LbestTemp;
            freject=f;
            Lreject=loc;
        end
    end
end
if listCounter~=1,
    [fbestNum,dimF]=size(fbest);
    [value index]=sort(fbest(:,1));
    sortedFbest=fbest(index,:);
    sortedLbest=Lbest(index,:);
    fbest=sortedFbest;
    Lbest=sortedLbest;
    fbest1=fbest(1:(fbestNum-1),:);
    fbest2=fbest(2:fbestNum,:);
    deltaf=abs(fbest2-fbest1);
    if areaType==1,
        area=ones(fbestNum-1,1);
        for i=1:dimF,
            area=deltaf(:,i).*area;
        end
    else
        area=zeros(fbestNum-1,1);
        for i=1:dimF,
            area=deltaf(:,i).^2+area;
        end
    end
    [MaxArea indexMax]=max(area);
    Lmax=[Lbest(indexMax,:) ; Lbest(indexMax+1,:)];
    Lbound=[];
    if Lbest(1,:)~=Lmax(1,:),
        Lbound=[Lbound;Lbest(1,:)];
    end
    if Lbest(fbestNum,:)~=Lmax(2,:),
        Lbound=[Lbound;Lbest(fbestNum,:)];
    end
end
if FirstIter==1,
    freject=[]; Lreject=[]; Sreject=1; area=[];
end

%==========================================================================
function z=dominate(x,y)
z1=all(x<=y);
z2=max(x<y);
z=z1*z2;
%==========================================================================
function [mapfit Lbest fbest loc V]=Ranking(f,Lbest,fbest,loc,V,Nmass,Nelit,Nlist,iter,max_it),

persistent listCounterI listCounter


fremind=[]; %rejected particles from archiving
Lremind=[];
Sremind=[];
mapfit=[];
loc=loc';
loctemp=loc; %[mass_num,dimX]
[mass_num,dimX]=size(loc);
[fNum,dimF]=size(f);
Vtemp=V';V=[];
for i=1:Nmass,
    [fbest Lbest freject Lreject Lmax Lbound listCounter Sreject area]=ParetoArchive(f(i,:),loctemp(i,:),fbest,Lbest,Nlist,listCounter,dimF);
    listCounterI(iter)=listCounter;
    listCount=listCounter;
    fremind=[fremind ; freject]; Lremind=[Lremind ; Lreject]; Sremind=[Sremind ; Sreject];
end

clear loc
[n,m]=size(Lmax);[n1,m1]=size(Lbound); k=n+n1;
if k==0,
    mapfit(1)=0; loc(1,:)=Lbest; V(1,:)=zeros(1,dimX); k=1;
else
    mapfit(1:k)=0; loc(1:k,:)=[Lmax ; Lbound]; V(1:k,:)=zeros(k,m);%elitisms must be fixed because not to loss their best positions
end

K=k;

if iter>1,
    Nelitism=round(Nelit*Nmass);
else
    Nelitism=1;
end
MaxBoundRank1=1;MaxBoundRank2=1;
MaxBoundRank=MaxBoundRank1+(MaxBoundRank2-MaxBoundRank1)/max_it*iter;

for i=2:Nlist-1,
    if mod(i,round(listCounterI(iter)/Nelitism))==0 && i<=listCounterI(iter),        
        if any(Lbest(i,:)~=Lmax(1,:)) && any(Lbest(i,:)~=Lmax(2,:)),
            mapfit(k+1)=MaxBoundRank; %start rank for other particles
            loc(k+1,:)=Lbest(i,:);
            V(k+1,:)=zeros(1,m);
            k=k+1;
        end
    end
end

K=[K k];

ElitRank=1;
InitialRank=mapfit(k)+1+ElitRank;
fPop=f;
locPop=loctemp;
VPop=Vtemp;
    
[fsort Lsort Vsort]=NDsorting(fPop,locPop,VPop,InitialRank);
[fsort,index]=sort(fsort,'descend'); Lsort=Lsort(index,:); Vsort=Vsort(index,:);
mapfit(k+1:Nmass)=fsort(k+1:Nmass);
loc(k+1:Nmass,:)=Lsort(k+1:Nmass,:);
V(k+1:Nmass,:)=Vsort(k+1:Nmass,:);
loc=loc'; V=V';

%==========================================================================
function [frank Lsort Vsort]=NDsorting(fremind,Lremind,Vremind,InitialRank);
InitialRank=InitialRank+1;
rank=InitialRank;j=1;fremindTemp=[];LremindTemp=[];VremindTemp=[];
[RemindNum,dimF]=size(fremind);
firstLayer=0;
while RemindNum~=0,
    [f1,index]=sort(fremind(:,1)); %sorting versus f1
    Lremind=Lremind(index,:);
    Vremind=Vremind(index,:);
    f2=fremind(index,2);
    if dimF==3,
        f3=fremind(index,3);
    end
    k=1;
    for i=1:RemindNum,
        if dimF==2,
            cond=f2(i)<=min(f2(1:i));
        elseif dimF==3,
            cond=f2(i)<=min(f2(1:i)) || f3(i)<=min(f3(1:i));
        end
        
        if cond;
            frank(j)=rank;
            fsort(j,:)=[f1(i) f2(i)];
            Lsort(j,:)=Lremind(i,:);
            Vsort(j,:)=Vremind(i,:);
            j=j+1;
        else
            if dimF==2,
                fremindTemp(k,:)=[f1(i) f2(i)];
            elseif dimF==3,
                fremindTemp(k,:)=[f1(i) f2(i) f3(i)];
            end
            LremindTemp(k,:)=Lremind(i,:);
            VremindTemp(k,:)=Vremind(i,:);
            k=k+1;
        end
    end
    if firstLayer==0,
        [fbestNum,dimF]=size(fsort);
        [value index]=sort(fsort(:,1));
        fbest1=fsort(1:(fbestNum-1),:);
        fbest2=fsort(2:fbestNum,:);
        deltaf=abs(fbest2-fbest1);
        area=zeros(fbestNum-1,1);
        for i=1:dimF,
            area=deltaf(:,i).^2+area;
        end
        [MaxArea indexMax]=max(area);
        frank(indexMax)=frank(indexMax)-1;
        frank(indexMax+1)=frank(indexMax+1)-1;
        fFirstLayer=fsort;
        firstLayer=1;
    end
    rank=rank+1;
    fremind=[];Lremind=[];Vremind=[];
    fremind=fremindTemp; Lremind=LremindTemp; Vremind=VremindTemp;
    fremindTemp=[];LremindTemp=[]; VremindTemp=[];
    [RemindNum,dimF]=size(fremind);
end
%==========================================================================
%==========================================================================

